@@ -109,8 +109,9 @@ sub create_table {
     $sql->{table} = '';
     $ax->reset_sql( $sql );
     my $skip_to = '';
+    my $count_table_name_loop = 0;
 
-    CREATE_TABLE: while ( 1 ) {
+    GET_CONTENT: while ( 1 ) {
         $sf->{i}{stmt_types} = [ 'Create_table', 'Insert' ];
         # first use of {stmt_types} in get_content/from_col_by_col
         my $ok = $gc->get_content( $sql, $skip_to );
@@ -118,20 +119,17 @@ sub create_table {
             return;
         }
         $skip_to = '';
-        my $count_table_name_loop = 0;
-        my $sheet_count = 0;
-        my $file_fs = $sf->{i}{gc}{file_fs};
-        if ( exists $sf->{i}{S_R}{$file_fs}{sheet_count} ) {
-            $sheet_count = $sf->{i}{S_R}{$file_fs}{sheet_count};
-        }
 
-        TABLE_NAME: while ( 1 ) {
-            my $no_default_table_name;
-            if ( $count_table_name_loop > 1 ) {
-                $no_default_table_name = 1;
-            }
-            my $ok_table_name = $sf->__set_table_name( $sql, $no_default_table_name ); # first time print_sql
-            if ( ! $ok_table_name ) {
+        my $sheet_count = 1;
+        my $file_fs = $sf->{i}{gc}{file_fs};
+        if ( exists $sf->{i}{ss}{$file_fs}{sheet_count} ) {
+            $sheet_count = $sf->{i}{ss}{$file_fs}{sheet_count};
+        }
+        my $table_name = ''; # table_name memory
+
+        GET_TABLE_NAME: while ( 1 ) {
+            $table_name = $sf->__get_table_name( $sql, $table_name, $count_table_name_loop ); # first time print_sql
+            if ( ! $table_name ) {
                 if ( $sf->{o}{insert}{enable_input_filter} ) {
                     $skip_to = 'FILTER';
                 }
@@ -141,60 +139,340 @@ sub create_table {
                 else {
                     $skip_to = 'GET_DATA';
                 }
-                next CREATE_TABLE;
+                $count_table_name_loop = 0;
+                next GET_CONTENT;
             }
+            my $bu_first_row = [ @{$sql->{insert_into_args}[0]} ];
+            my $orig_row_count = @{$sql->{insert_into_args}};
 
-            SET_COLUMNS: while ( 1 ) {
-                my $ok_columns = $sf->__set_columns( $sql );
-                if ( ! $ok_columns ) {
+            GET_COLUMN_NAMES: while ( 1 ) {
+                if ( $orig_row_count - 1 == @{$sql->{insert_into_args}} ) {
+                    unshift @{$sql->{insert_into_args}}, $bu_first_row;
+                }
+                $sql->{create_table_cols} = [];
+                $sql->{insert_into_cols}  = [];
+                my $header_row = $sf->__get_column_names( $sql );
+                if ( ! $header_row ) {
                     $sql->{table} = '';
-                    if ( exists $sf->{i}{ct}{shifted_header} ) {
-                        unshift @{$sql->{insert_into_args}}, delete $sf->{i}{ct}{shifted_header};
+                    if ( $orig_row_count - 1 == @{$sql->{insert_into_args}[0]} ) {
+                        unshift @{$sql->{insert_into_args}}, $bu_first_row;
                     }
                     $count_table_name_loop++;
-                    next TABLE_NAME;
+                    next GET_TABLE_NAME;
                 }
                 $count_table_name_loop = 0;
-                my $ok_create_table = $sf->__create( $sql, 'table' );
-                if ( ! defined $ok_create_table ) {
-                    if ( exists $sf->{i}{ct}{shifted_header} ) {
-                        unshift @{$sql->{insert_into_args}}, delete $sf->{i}{ct}{shifted_header};
+
+                AUTO_INCREMENT: while( 1 ) {
+                    $sql->{create_table_cols} = [ @$header_row ];  # not quoted
+                    $sql->{insert_into_cols}  = [ @$header_row ];  # not quoted
+                    my $continue = $sf->__autoincrement_column( $sql);
+                    if ( ! $continue ) {
+                        next GET_COLUMN_NAMES;
                     }
-                    next SET_COLUMNS;
-                }
-                if ( ! $ok_create_table ) {
-                    return;
-                }
-                if ( @{$sql->{insert_into_args}} ) {
-                    my $ok_insert = $sf->__insert_data( $sql );
-                    if ( ! $ok_insert ) {
-                        require App::DBBrowser::DropTable;
-                        my $dt = App::DBBrowser::DropTable->new( $sf->{i}, $sf->{o}, $sf->{d} );
-                        my $drop_ok = $dt->__drop( $sql, 'table' );
-                        if ( ! $drop_ok ) {
-                            return;
+                    my @bu_orig_create_table_cols = @{$sql->{create_table_cols}};
+                    my $column_names = []; # column_names memory
+
+                    EDIT_COLUMN_NAMES: while( 1 ) {
+                        $column_names = $sf->__edit_column_names( $sql, $column_names );
+                        if ( ! $column_names ) {
+                            $sql->{create_table_cols} = [ @bu_orig_create_table_cols ];
+                            next AUTO_INCREMENT if $sf->{col_auto};
+                            next GET_COLUMN_NAMES;
                         }
-                        $sf->{i}{stmt_types} = [ 'Create_table', 'Insert' ];
-                        if ( exists $sf->{i}{ct}{shifted_header} ) {
-                            unshift @{$sql->{insert_into_args}}, delete $sf->{i}{ct}{shifted_header};
+                        if ( any { ! length } @{$sql->{create_table_cols}} ) {
+                            # Choose
+                            $tc->choose(
+                                [ 'Column with no name!' ],
+                                { prompt => 'Continue with ENTER', keep => 1 }
+                            );
+                            next EDIT_COLUMN_NAMES;
+                        }
+                        my @duplicates = duplicates map { lc } @{$sql->{create_table_cols}};
+                        if ( @duplicates ) {
+                            # Choose
+                            $tc->choose(
+                                [ 'Duplicate column name!' ],
+                                { prompt => 'Continue with ENTER', keep => 1 }
+                            );
+                            next EDIT_COLUMN_NAMES;
+                        }
+                        my @bu_edited_create_table_cols = @{$sql->{create_table_cols}};
+                        my $data_types = {}; # data_types memory
+
+                        EDIT_COLUMN_TYPES: while( 1 ) {
+                            $data_types = $sf->__edit_column_types( $sql, $data_types );
+                            if ( ! $data_types ) {
+                                $sql->{create_table_cols} = [ @bu_orig_create_table_cols ];
+                                next EDIT_COLUMN_NAMES;
+                            }
+
+                            # CREATE_TABLE
+                            my $ok_create_table = $sf->__create( $sql, 'table' );
+                            if ( ! defined $ok_create_table ) {
+                                $sql->{create_table_cols} = [ @bu_edited_create_table_cols ];
+                                $sql->{insert_into_cols}  = [];
+                                next EDIT_COLUMN_TYPES;
+                            }
+                            if ( ! $ok_create_table ) {
+                                return;
+                            }
+                            if ( @{$sql->{insert_into_args}} ) {
+
+                                # INSERT_DATA
+                                my $ok_insert = $sf->__insert_data( $sql );
+                                if ( ! $ok_insert ) {
+                                    require App::DBBrowser::DropTable;
+                                    my $dt = App::DBBrowser::DropTable->new( $sf->{i}, $sf->{o}, $sf->{d} );
+                                    my $drop_ok = $dt->__drop( $sql, 'table' );
+                                    if ( ! $drop_ok ) {
+                                        return;
+                                    }
+                                    $sf->{i}{stmt_types} = [ 'Create_table', 'Insert' ];
+                                    next GET_COLUMN_NAMES;
+                                }
+                            }
+                            if ( $sheet_count > 1 ) {
+                                $skip_to = 'PARSE';
+                            }
+                            else {
+                                $skip_to = 'GET_DATA';
+                            }
+                            next GET_CONTENT;
                         }
-                        next SET_COLUMNS;
                     }
                 }
-                if ( $sheet_count > 1 ) {
-                    $skip_to = 'PARSE';
-                    next CREATE_TABLE;
-                }
-                delete $sf->{i}{gc}{old_dir_idx}            if exists $sf->{i}{gc}{old_dir_idx};
-                delete $sf->{i}{gc}{files_fs_in_chosen_dir} if exists $sf->{i}{gc}{files_fs_in_chosen_dir};
-                delete $sf->{i}{gc}{old_file_idx}           if exists $sf->{i}{gc}{old_file_idx};
-                return 1;
             }
         }
     }
 }
 
 
+sub __get_table_name {
+    my ( $sf, $sql, $table_name, $count_table_name_loop ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my $tf = Term::Form->new( $sf->{i}{tf_default} );
+    $ax->print_sql_info( $ax->get_sql_info( $sql ) );
+
+    while ( 1 ) {
+        my $file_info;
+        if ( $sf->{i}{gc}{source_type} eq 'file' ) {
+            my $file_fs = $sf->{i}{gc}{file_fs};
+            my $file_name = basename decode( 'locale_fs', $file_fs );
+            $file_info = sprintf "File: '%s'", $file_name;
+            if ( ! length $table_name ) {
+                my $sheet_name = $sf->{i}{ss}{$file_fs}{sheet_name};
+                if ( defined $sheet_name && length $sheet_name ) {
+                    if ( $sf->{i}{ss}{$file_fs}{sheet_count} > 1 ) {
+                        $table_name = $sheet_name;
+                    }
+                    else {
+                        $file_name =~ s/\.[^.]{1,4}\z//;
+                        $table_name = $file_name . '_' . $sheet_name;
+                    }
+                }
+                else {
+                    ( $table_name = $file_name ) =~ s/\.[^.]{1,4}\z//;
+                }
+                $table_name =~ s/ /_/g;
+            }
+        }
+        $sf->{i}{occupied_term_height} = 2; # readline, trailing empty line
+        if ( $file_info ) {
+            $sf->{i}{occupied_term_height} += 1;
+        }
+        if ( $count_table_name_loop > 1 ) {
+            $table_name = '';
+        }
+        my $info = $ax->get_sql_info( $sql ) . ( $file_info ? "\n" . $file_info : '' );
+        # Readline
+        $table_name = $tf->readline(
+            'Table name: ',
+            { info => $info, default => $table_name }
+        );
+        $ax->print_sql_info( $info );
+        if ( ! length $table_name ) {
+            return;
+        }
+        $sql->{table} = $ax->quote_table( [ undef, $sf->{d}{schema}, $table_name ] );
+        if ( none { $sql->{table} eq $ax->quote_table( $sf->{d}{tables_info}{$_} ) } keys %{$sf->{d}{tables_info}} ) {
+            return $table_name;
+        }
+        $sf->{i}{occupied_term_height} = 4; # prompt, $menu, trailing empty line
+        my $prompt = "Table $sql->{table} already exists.";
+        my $menu = [ undef, '  New name' ];
+        $info = $ax->get_sql_info( $sql );
+        # Choose
+        my $chosen = $tc->choose(
+            $menu,
+            { %{$sf->{i}{lyt_v}}, info => $info, prompt => $prompt, keep => scalar( @$menu ) }
+        );
+        $ax->print_sql_info( $info );
+        if ( ! defined $chosen ) {
+            return;
+        }
+    }
+}
+
+
+sub __get_column_names {
+    my ( $sf, $sql ) = @_;
+    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my ( $first_row, $user_input ) = ( '- YES', '- NO' );
+    my @pre = ( undef );
+    my $menu = [ @pre, $first_row, $user_input ];
+    my $header_row;
+    $sf->{i}{occupied_term_height} = @$menu + 2; # + 2 for prompt and empty line
+    my $info = $ax->get_sql_info( $sql );
+    # Choose
+    my $chosen = $tc->choose(
+        $menu,
+        { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Use the first Data Row as the Table Header:', undef => '  <=',
+          keep => scalar( @$menu ) }
+    );
+    $ax->print_sql_info( $info );
+    if ( ! defined $chosen ) {
+        return;
+    }
+    elsif ( $chosen eq $first_row ) {
+        $header_row = shift @{$sql->{insert_into_args}};
+    }
+    else {
+        my $c = 0;
+        $header_row = $sf->__generic_header_row( scalar @{$sql->{insert_into_args}->[0]} );
+    }
+    return $header_row;
+}
+
+
+sub __autoincrement_column {
+    my ( $sf, $sql ) = @_;
+    my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
+    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    $sf->{col_auto} = '';
+    if ( $sf->{constraint_auto} = $plui->primary_key_autoincrement_constraint( $sf->{d}{dbh} ) ) {
+        $sf->{col_auto} = $sf->{o}{create}{autoincrement_col_name};
+    }
+    if ( $sf->{col_auto} ) {
+        my ( $no, $yes ) = ( '- NO ', '- YES' );
+        my $menu = [ undef, $yes, $no  ];
+        $sf->{i}{occupied_term_height} = @$menu + 2; # + 2 for prompt and empty line
+        my $info = $ax->get_sql_info( $sql );
+        # Choose
+        my $chosen = $tc->choose(
+            $menu,
+            { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Add an AUTO INCREMENT column:', undef => '  <=',
+              keep => scalar( @$menu )  }
+        );
+        $ax->print_sql_info( $info );
+        if ( ! defined $chosen ) {
+            return;
+        }
+        elsif ( $chosen eq $no ) {
+            $sf->{col_auto} = '';
+        }
+        else {
+            unshift @{$sql->{create_table_cols}}, $sf->{col_auto};
+        }
+    }
+    return 1;
+}
+
+
+sub __edit_column_names {
+    my ( $sf, $sql, $column_names ) = @_;
+    my $tf = Term::Form->new( $sf->{i}{tf_default} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $col_number = 0;
+    my $db = $sf->{d}{db};
+    my $fields;
+    if ( @$column_names ) {
+        $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @$column_names ];
+    }
+    else {
+        $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @{$sql->{create_table_cols}} ];
+    }
+    $sf->{i}{occupied_term_height} = 3 + @{$sql->{create_table_cols}};
+    my $info = $ax->get_sql_info( $sql );
+    # Fill_form
+    my $form = $tf->fill_form(
+        $fields,
+        { info => $info, prompt => 'Edit column names:', auto_up => 2,
+          confirm => $sf->{i}{confirm}, back => $sf->{i}{back} . '   ' }
+    );
+    $ax->print_sql_info( $info );
+    if ( ! defined $form ) {
+        return;
+    }
+    $column_names = $sql->{create_table_cols} = [ map { $_->[1] } @$form ]; # not quoted
+    if ( length $sf->{col_auto} ) {
+        $sf->{col_auto} = $sql->{create_table_cols}[0];
+    }
+    return $column_names;
+}
+
+
+sub __edit_column_types {
+    my ( $sf, $sql, $data_types ) = @_;
+    my $tf = Term::Form->new( $sf->{i}{tf_default} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $unquoted_table_cols = [ @{$sql->{create_table_cols}} ];
+    $sql->{create_table_cols} = $ax->quote_simple_many( $sql->{create_table_cols} ); # now quoted
+    $sql->{insert_into_cols} = [ @{$sql->{create_table_cols}} ];
+    if ( length $sf->{col_auto} ) {
+        shift @{$sql->{insert_into_cols}};
+    }
+    my $fields;
+    if ( ! %$data_types && $sf->{o}{create}{data_type_guessing} ) {
+        $sf->{i}{occupied_term_height} = 4 + @{$sql->{insert_into_cols}}; # busy string at the height of the prompt
+        $ax->print_sql_info( $ax->get_sql_info( $sql ), 'Column data types: guessing ... ' );
+        require SQL::Type::Guess;
+        my $g = SQL::Type::Guess->new();
+        my $header = $sql->{insert_into_cols}; #
+        my $table  = $sql->{insert_into_args};
+        my @aoh;
+        for my $row ( @$table ) {
+            push @aoh, {
+                map { $header->[$_] => $row->[$_] } 0 .. $#{$row}
+            };
+        }
+        $g->guess( @aoh );
+        my $tmp = $g->column_type;
+        $data_types = { map { $_ => uc( $tmp->{$_} ) } keys %$tmp };
+    }
+    if ( defined $data_types ) {
+        $fields = [ map { [ $_, $data_types->{$_} ] } @{$sql->{insert_into_cols}} ];
+    }
+    else {
+        $fields = [ map { [ $_, '' ] } @{$sql->{insert_into_cols}} ];
+    }
+    my $read_only = [];
+    if ( length $sf->{col_auto} ) {
+        unshift @$fields, [ $ax->quote_col_qualified( [ $sf->{col_auto} ] ), $sf->{constraint_auto} ];
+        $read_only = [ 0 ];
+    }
+    $sf->{i}{occupied_term_height} = 3 + @$fields; # prompt, back, confirm and fiels # 4 with trailing empty line
+    my $info = $ax->get_sql_info( $sql );
+    # Fill_form
+    my $col_name_and_type = $tf->fill_form(
+        $fields,
+        { info => $info, prompt => 'Column data types:', auto_up => 2, read_only => $read_only,
+          confirm => $sf->{i}{confirm}, back => $sf->{i}{back} . '   ' }
+    );
+    $ax->print_sql_info( $info );
+    if ( ! $col_name_and_type ) {
+        return;
+    }
+    else {
+        no warnings 'uninitialized';
+        $sql->{create_table_cols} = [ map { join ' ', @$_ }  @$col_name_and_type ];
+    }
+    return $data_types;
+}
+
+
 sub __create {
     my ( $sf, $sql, $type ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
@@ -255,362 +533,12 @@ sub __insert_data {
 }
 
 
-sub __set_table_name {
-    my ( $sf, $sql, $no_default_table_name ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my $tf = Term::Form->new( $sf->{i}{tf_default} );
-    my $table;
-    my $c = 0;
-    $ax->print_sql_info( $ax->get_sql_info( $sql ) );
-
-    while ( 1 ) {
-        my $file_info;
-        if ( $sf->{i}{gc}{source_type} eq 'file' ) {
-            my $file_fs = $sf->{i}{gc}{file_fs};
-            my $file_name = basename decode( 'locale_fs', $file_fs );
-            $file_info = sprintf "File: '%s'", $file_name;
-            my $sheet_name = $sf->{i}{S_R}{$file_fs}{sheet_name};
-            if ( defined $sheet_name && length $sheet_name ) {
-                if ( $sf->{i}{S_R}{$file_fs}{sheet_count} > 1 ) {
-                    $sf->{i}{ct}{default_table_name} = $sheet_name;
-                }
-                else {
-                    $file_name =~ s/\.[^.]{1,4}\z//;
-                    $sf->{i}{ct}{default_table_name} = $file_name . '_' . $sheet_name;
-                }
-            }
-            else {
-                ( $sf->{i}{ct}{default_table_name} = $file_name ) =~ s/\.[^.]{1,4}\z//;
-            }
-            $sf->{i}{ct}{default_table_name} =~ s/ /_/g;
-        }
-        $sf->{i}{occupied_term_height} = 2; # readline, trailing empty line
-        if ( $file_info ) {
-            $sf->{i}{occupied_term_height} += 1;
-        }
-        my $info = $ax->get_sql_info( $sql ) . ( $file_info ? "\n" . $file_info : '' );
-        # Readline
-        $table = $tf->readline(
-            'Table name: ',
-            { info => $info, default => $no_default_table_name ? undef : $sf->{i}{ct}{default_table_name} }
-        );
-        $ax->print_sql_info( $info );
-        if ( ! length $table ) {
-            return;
-        }
-        $sf->{i}{ct}{default_table_name} = $table;
-        $sql->{table} = $ax->quote_table( [ undef, $sf->{d}{schema}, $table ] );
-        if ( none { $sql->{table} eq $ax->quote_table( $sf->{d}{tables_info}{$_} ) } keys %{$sf->{d}{tables_info}} ) {
-            return 1;
-        }
-        $sf->{i}{occupied_term_height} = 4; # prompt, $menu, trailing empty line
-        my $prompt = "Table $sql->{table} already exists.";
-        my $menu = [ undef, '  New name' ];
-        $info = $ax->get_sql_info( $sql );
-        # Choose
-        my $chosen = $tc->choose(
-            $menu,
-            { %{$sf->{i}{lyt_v}}, info => $info, prompt => $prompt, keep => scalar( @$menu ) }
-        );
-        $ax->print_sql_info( $info );
-        if ( ! defined $chosen ) {
-            return;
-        }
-    }
-}
-
-
-sub __set_columns {
-    my ( $sf, $sql ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    if ( exists $sf->{i}{ct}{shifted_header} ) {
-        delete $sf->{i}{ct}{shifted_header};
-    }
-
-    HEADER_ROW: while ( 1 ) {
-        if ( exists $sf->{i}{ct}{shifted_header} ) {
-            unshift @{$sql->{insert_into_args}}, delete $sf->{i}{ct}{shifted_header};
-        }
-        $sql->{create_table_cols} = [];
-        $sql->{insert_into_cols}  = [];
-        my $header_row = $sf->__header_row( $sql );
-        if ( ! $header_row ) {
-            return;
-        }
-
-        AI_COL: while ( 1 ) {
-            $sql->{create_table_cols} = [ @$header_row ];  # not quoted
-            $sql->{insert_into_cols}  = [ @$header_row ];  # not quoted
-            my $continue = $sf->__autoincrement_column( $sql);
-            if ( ! $continue ) {
-                next HEADER_ROW;
-            }
-            my @bu_create_table_cols = @{$sql->{create_table_cols}};
-
-            COL_NAMES: while ( 1 ) {
-                my $ok_names = $sf->__column_names( $sql );
-                if ( ! $ok_names ) {
-                    $sql->{create_table_cols} = [ @bu_create_table_cols ];
-                    next AI_COL if $sf->{col_auto};
-                    next HEADER_ROW;
-                }
-                if ( any { ! length } @{$sql->{create_table_cols}} ) {
-                    # Choose
-                    $tc->choose(
-                        [ 'Column with no name!' ],
-                        { prompt => 'Continue with ENTER', keep => 1 }
-                    );
-                    next COL_NAMES;
-                }
-                my @duplicates = duplicates map { lc } @{$sql->{create_table_cols}};
-                if ( @duplicates ) {
-                    # Choose
-                    $tc->choose(
-                        [ 'Duplicate column name!' ],
-                        { prompt => 'Continue with ENTER', keep => 1 }
-                    );
-                    next COL_NAMES;
-                }
-                my @bu_cols_with_name = @{$sql->{create_table_cols}};
-
-                DATA_TYPES: while ( 1 ) {
-                    my $ok_data_types = $sf->__data_types( $sql );
-                    if ( ! $ok_data_types ) {
-                        $sql->{create_table_cols} = [ @bu_cols_with_name ];
-                        next COL_NAMES;
-                    }
-                    last DATA_TYPES;
-                }
-                last COL_NAMES;
-            }
-            last AI_COL;
-        }
-        last HEADER_ROW;
-    }
-    return 1;
-}
-
-
-sub __header_row {
-    my ( $sf, $sql ) = @_;
-    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my ( $first_row,        $user_input        ) = ( '- YES',                   '- NO'                   );
-    my ( $first_row_edited, $user_input_edited ) = ( '- YES, edited as before', '- NO, edited as before' );                 #
-    my @pre = ( undef );
-    my $menu = [ @pre, $first_row, $user_input ];
-    my $prev_edited_header;                                                                                                 #
-    if ( exists $sf->{i}{ct}{prev_header_info} ) {                                                                          #
-        ( my $prev_orig_header, $prev_edited_header, my $header_is_first_data_row ) = @{$sf->{i}{ct}{prev_header_info}};    #
-        if ( $header_is_first_data_row ) {                                                                                  #
-            my $add_to_menu = 1;                                                                                            #
-            for my $i ( 0 .. $#$prev_orig_header ) {                                                                        #
-                if ( $prev_orig_header->[$i] ne $sql->{insert_into_args}[0][$i] ) {                                         #
-                    $add_to_menu = 0;                                                                                       #
-                    last;                                                                                                   #
-                }                                                                                                           #
-            }                                                                                                               #
-            if ( $add_to_menu ) {                                                                                           #
-                splice @$menu, 2, 0, $first_row_edited;                                                                     #
-            }                                                                                                               #
-        }                                                                                                                   #
-        else {                                                                                                              #
-            if ( @$prev_orig_header == @{$sql->{insert_into_args}[0]} ) {                                                   #
-                push @$menu, $user_input_edited;                                                                            #
-            }                                                                                                               #
-        }                                                                                                                   #
-    }                                                                                                                       #
-    my $header_row;
-    $sf->{i}{occupied_term_height} = @$menu + 2; # + 2 for prompt and empty line
-    my $info = $ax->get_sql_info( $sql );
-    # Choose
-    my $chosen = $tc->choose(
-        $menu,
-        { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Use the first Data Row as the Table Header:', undef => '  <=',
-          keep => scalar( @$menu ) }
-    );
-    $ax->print_sql_info( $info );
-    if ( ! defined $chosen ) {
-        return;
-    }
-    elsif ( $chosen eq $first_row ) {
-        $sf->{i}{ct}{shifted_header} = shift @{$sql->{insert_into_args}};
-        $header_row = [ @{$sf->{i}{ct}{shifted_header}} ];
-    }
-    elsif ( $chosen eq $first_row_edited ) {                                #
-        $sf->{i}{ct}{shifted_header} = shift @{$sql->{insert_into_args}};   #
-        $header_row = $prev_edited_header;                                  #
-    }                                                                       #
-    elsif ( $chosen eq $user_input_edited ) {                               #
-        $header_row = $prev_edited_header;                                  #
-    }                                                                       #
-    else {
-        my $c = 0;
-        $header_row = $sf->__generic_header_row( scalar @{$sql->{insert_into_args}->[0]} );
-    }
-    return $header_row;
-}
-
-
-sub __autoincrement_column {
-    my ( $sf, $sql ) = @_;
-    my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
-    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    $sf->{col_auto} = '';
-    if ( $sf->{constraint_auto} = $plui->primary_key_autoincrement_constraint( $sf->{d}{dbh} ) ) {
-        $sf->{col_auto} = $sf->{o}{create}{autoincrement_col_name};
-    }
-    if ( $sf->{col_auto} ) {
-        my ( $no, $yes ) = ( '- NO ', '- YES' );
-        my $menu = [ undef, $yes, $no  ];
-        $sf->{i}{occupied_term_height} = @$menu + 2; # + 2 for prompt and empty line
-        my $info = $ax->get_sql_info( $sql );
-        # Choose
-        my $chosen = $tc->choose(
-            $menu,
-            { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Add an AUTO INCREMENT column:', undef => '  <=',
-              keep => scalar( @$menu )  }
-        );
-        $ax->print_sql_info( $info );
-        if ( ! defined $chosen ) {
-            return;
-        }
-        elsif ( $chosen eq $no ) {
-            $sf->{col_auto} = '';
-        }
-        else {
-            unshift @{$sql->{create_table_cols}}, $sf->{col_auto};
-        }
-    }
-    return 1;
-}
-
-sub __column_names {
-    my ( $sf, $sql ) = @_;
-    my $tf = Term::Form->new( $sf->{i}{tf_default} );
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $col_number = 0;
-    my $db = $sf->{d}{db};
-    my $fields = [ map { [ ++$col_number, defined $_ ? "$_" : '' ] } @{$sql->{create_table_cols}} ];
-    $sf->{i}{occupied_term_height} = 3 + @{$sql->{create_table_cols}};
-    my $info = $ax->get_sql_info( $sql );
-    # Fill_form
-    my $form = $tf->fill_form(
-        $fields,
-        { info => $info, prompt => 'Edit column names:', auto_up => 2,
-          confirm => $sf->{i}{confirm}, back => $sf->{i}{back} . '   ' }
-    );
-    $ax->print_sql_info( $info );
-    if ( ! defined $form ) {
-        return;
-    }
-    my ( $orig_header, $header_is_first_data_row );                                                     #
-    if ( exists $sf->{i}{ct}{shifted_header} ) {                                                        #
-        $orig_header = [ @{$sf->{i}{ct}{shifted_header}} ];                                             #
-        $header_is_first_data_row = 1;                                                                  #
-    }                                                                                                   #
-    else {                                                                                              #
-        $orig_header = $sf->__generic_header_row( scalar @{$sql->{insert_into_args}->[0]} );            #
-        $header_is_first_data_row = 0;                                                                  #
-    }                                                                                                   #
-    my $edited_header = [ map { $_->[1] } @$form ];                                                     #
-    if ( length $sf->{col_auto} ) {                                                                     #
-        shift @$edited_header;                                                                          #
-    }                                                                                                   #
-    my $diff_between_orig_and_edited_header = 0;                                                        #
-    for my $i ( 0 .. $#$orig_header ) {                                                                 #
-        if ( $orig_header->[$i] ne $edited_header->[$i] ) {                                             #
-            $diff_between_orig_and_edited_header = 1;                                                   #
-            last;                                                                                       #
-        }                                                                                               #
-    }                                                                                                   #
-    if ( $diff_between_orig_and_edited_header ) {                                                       #
-        $sf->{i}{ct}{prev_header_info} = [ $orig_header, $edited_header, $header_is_first_data_row ];   #
-    }                                                                                                   #
-    elsif ( exists $sf->{i}{ct}{prev_header_info} ) {                                                   #
-        delete $sf->{i}{ct}{prev_header_info};                                                          #
-    }                                                                                                   #
-    $sql->{create_table_cols} = [ map { $_->[1] } @$form ]; # not quoted
-    if ( length $sf->{col_auto} ) {
-        $sf->{col_auto} = $sql->{create_table_cols}[0];
-    }
-    return 1;
-}
-
 sub __generic_header_row {
     my ( $sf, $col_count ) = @_;
     my $c = 0;
     return [ map { 'c' . $c++ } 1 .. $col_count ];
 }
 
-sub __data_types {
-    my ( $sf, $sql ) = @_;
-    my $tf = Term::Form->new( $sf->{i}{tf_default} );
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $unquoted_table_cols = [ @{$sql->{create_table_cols}} ];
-    $sql->{create_table_cols} = $ax->quote_simple_many( $sql->{create_table_cols} ); # now quoted
-    $sql->{insert_into_cols} = [ @{$sql->{create_table_cols}} ];
-    if ( length $sf->{col_auto} ) {
-        shift @{$sql->{insert_into_cols}};
-    }
-    my $data_types;
-    my $fields;
-    if ( $sf->{o}{create}{data_type_guessing} ) {
-        $sf->{i}{occupied_term_height} = 4 + @{$sql->{insert_into_cols}}; # busy string at the height of the prompt
-        $ax->print_sql_info( $ax->get_sql_info( $sql ), 'Column data types: guessing ... ' );
-        $data_types = $sf->__guess_data_type( $sql );
-    }
-    if ( defined $data_types ) {
-        $fields = [ map { [ $_, $data_types->{$_} ] } @{$sql->{insert_into_cols}} ];
-    }
-    else {
-        $fields = [ map { [ $_, '' ] } @{$sql->{insert_into_cols}} ];
-    }
-    my $read_only = [];
-    if ( length $sf->{col_auto} ) {
-        unshift @$fields, [ $ax->quote_col_qualified( [ $sf->{col_auto} ] ), $sf->{constraint_auto} ];
-        $read_only = [ 0 ];
-    }
-    $sf->{i}{occupied_term_height} = 3 + @$fields; # prompt, back, confirm and fiels # 4 with trailing empty line
-    my $info = $ax->get_sql_info( $sql );
-    # Fill_form
-    my $col_name_and_type = $tf->fill_form(
-        $fields,
-        { info => $info, prompt => 'Column data types:', auto_up => 2, read_only => $read_only,
-          confirm => $sf->{i}{confirm}, back => $sf->{i}{back} . '   ' }
-    );
-    $ax->print_sql_info( $info );
-    if ( ! $col_name_and_type ) {
-        return;
-    }
-    else {
-        no warnings 'uninitialized';
-        $sql->{create_table_cols} = [ map { join ' ', @$_ }  @$col_name_and_type ];
-    }
-    return 1;
-}
-
-
-sub __guess_data_type {
-    my ( $sf, $sql ) = @_;
-    require SQL::Type::Guess;
-    my $g = SQL::Type::Guess->new();
-    my $header = $sql->{insert_into_cols}; #
-    my $table  = $sql->{insert_into_args};
-    my @aoh;
-    for my $row ( @$table ) {
-        push @aoh, {
-            map { $header->[$_] => $row->[$_] } 0 .. $#{$row}
-        };
-    }
-    $g->guess( @aoh );
-    my $tmp = $g->column_type;
-    return { map { $_ => uc( $tmp->{$_} ) } keys %$tmp };
-}
-
 
 
 
